# Internationalization (I18N) - Language Translations

## Overview

This document provides a brief overview of changes needed when adding new languages to DataFormsJS. Three different repositories need to be updated for full site translations.

This document itself does NOT need to be translated rather it specifies what files and code should be updated.

The following setup is recommended for local development.

~~~
dataformsjs    {root-directory}
├── dataformsjs  [repository]
├── playground   [repository]
└── website      [repository]
~~~

If you need to install PHP see documents for (Windows, macOS, and Linux) from: https://www.fastsitephp.com/en/getting-started

Files are copied from English, Google Translate is used for initial translations, and then human translators fix errors from Google Translate.

Languages currently needed: Arabic, French, German, Italian, Japanese, Persian, Russian

Languages currently being tranlsated: Chinese Simplified (zh-CN), Chinese Traditional (zh-TW), Korean, Portuguese

## Generating the files

All needed files can be generated by using the php script once all 3 repositories are downloaded:

https://github.com/dataformsjs/website/blob/master/scripts/create-i18n-files.php

Simply change the value for `const LANG_COPY_TO = '{lang}';` on line 18 and run it.

## JSON Files

Files for specific languages end with `*.{lang}.json` and the main files for the main site is `_.{lang}.json`.

* `dataformsjs\examples\i18n`
* `website\public\i18n`

https://github.com/dataformsjs/dataformsjs/tree/master/examples/i18n

https://github.com/dataformsjs/website/tree/master/public/i18n

## Related HTML Files

https://github.com/dataformsjs/website/blob/master/app/Views/index.htm

See line #5: `data-i18n-locales="en,..."`. Once all JSON files are created copy the full text for the [data-i18n-locales] attribute and then find/replace globally in all files using a Code Editor. As of 12/2019 there are 30 instances in 12 files.

In the main file above search for `i18n-menu` and add the new language so it shows on the nav menus. There are two locations (one for desktop and one for mobile). Also search for `app.plugins.i18n.currentLocale` and make needed changes so the language links to the translated GitHub readme.

For React Files search for `const supportedLocales =` and then add the language to the array. As of 12/2019 there are 2 files that need to be updated.

The order of languages for the Nav Menu is based on alphabetical sort order, this can be determined by copying the text to Excel and sorting.

For a good link on lanuages used on the internet see: https://en.wikipedia.org/wiki/Languages_used_on_the_Internet

# Code Playground Template

Each language has it's own template directory/folder. Once a directory is created for a language then it can be used for to create sites. Text content and code comments are updated to the translated language.

`playground\app_data\template\{lang}`

https://github.com/dataformsjs/playground/tree/master/app_data/template

This can be done without a local setup, but if you would prefer to test the site locally refer to instructions in the readme of the playground repository.

https://github.com/dataformsjs/playground

## Main Site Readme

Some of this content is duplicated from the main home page so some content simply be copied after translating JSON files. The 'Paid Translators Needed' section should be manually deleted on the copied file.

`dataformsjs\docs\i18n-readme\README.{lang}.md`

https://github.com/dataformsjs/dataformsjs/tree/master/docs/i18n-readme

## Additional Readme updates

Readme files for the other repositories will have a single row added to the table at the top of the file. Copy and tanslate the English row.

https://github.com/dataformsjs/website/blob/master/README.md

https://github.com/dataformsjs/playground/blob/master/README.md

https://github.com/dataformsjs/static-files/blob/master/README.md

## Quick Reference Code

Code comments are updated to the translated language.

`website\app_data\example-code\quick-reference-{lang}.htm`

https://github.com/dataformsjs/website/tree/master/app_data/example-code

## Sitemap Update

Once content is ready the `sitemap.xml` file can be updated by copying the links in the English section and replacing to `/en/` text with the code for the translated language. In the future this may be automated but for now there are not that many links so it's quick to copy/paste/modify.

`website\public\sitemap.xml`
